/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.apache.isis.runtimes.dflt.remoting.transport.pipe;

import org.apache.isis.runtimes.dflt.remoting.common.exchange.Request;
import org.apache.isis.runtimes.dflt.remoting.common.exchange.ResponseEnvelope;
import org.apache.isis.runtimes.dflt.remoting.common.facade.ServerFacade;
import org.apache.log4j.Logger;

public class PipedServer {
    private static final Logger LOG = Logger.getLogger(PipedServer.class);

    private ServerFacade facade;
    private PipedConnection communication;

    public synchronized void run() {
        while (true) {
            final Request request = communication.getRequest();
            LOG.debug("client request: " + request);
            try {
                request.execute(facade);
                final ResponseEnvelope response = new ResponseEnvelope(request);
                LOG.debug("server response: " + response);
                communication.setResponse(response);
            } catch (final RuntimeException e) {
                communication.setException(e);
            } catch (final Exception e) {
                LOG.error("failure during request", e);
            }
        }
    }

    public synchronized void setConnection(final PipedConnection communication) {
        this.communication = communication;
    }

    public synchronized void setFacade(final ServerFacade facade) {
        this.facade = facade;
    }
}
